{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.ExternalFunction
       (
         ExternalFunction,
         ExternalFunctionClass(..),
         externalFunction,
         externalFunction',
         externalFunction_checkNode,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.ForeignToolsInstances ( )
import Casadi.Wrappers.Deleters
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, CppVecVec, CppVecVecVec,
                             StdString', CppBool' ) -- StdOstream'
import Casadi.Marshal ( CornerCase(..), Marshal(..) )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show ExternalFunction where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__ExternalFunction__checkNode" c_CasADi__ExternalFunction__checkNode
  :: Ptr ExternalFunction' -> IO (Ptr CppBool')
casADi__ExternalFunction__checkNode
  :: ExternalFunction -> IO Bool
casADi__ExternalFunction__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__ExternalFunction__checkNode x0' >>= (newForeignPtr c_delete_bool) >>= wrapReturn

-- classy wrapper
{-|
>Check if the pointer points towards a valid object.
-}
externalFunction_checkNode :: ExternalFunctionClass a => a -> IO Bool
externalFunction_checkNode x = casADi__ExternalFunction__checkNode (castExternalFunction x)


-- direct wrapper
foreign import ccall unsafe "CasADi__ExternalFunction__ExternalFunction" c_CasADi__ExternalFunction__ExternalFunction
  :: IO (Ptr ExternalFunction')
casADi__ExternalFunction__ExternalFunction
  :: IO ExternalFunction
casADi__ExternalFunction__ExternalFunction  =
  c_CasADi__ExternalFunction__ExternalFunction  >>= (newForeignPtr c_delete_CasADi__ExternalFunction) >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::ExternalFunction::ExternalFunction()
>------------------------------------------------------------------------
>
>CONSTRUCTORS:
>
>default constructor
>
>>  CasADi::ExternalFunction::ExternalFunction(const std::string &bin_name)
>------------------------------------------------------------------------
>
>Create an empty function.
-}
externalFunction :: IO ExternalFunction
externalFunction = casADi__ExternalFunction__ExternalFunction


-- direct wrapper
foreign import ccall unsafe "CasADi__ExternalFunction__ExternalFunction_TIC" c_CasADi__ExternalFunction__ExternalFunction_TIC
  :: Ptr StdString' -> IO (Ptr ExternalFunction')
casADi__ExternalFunction__ExternalFunction'
  :: String -> IO ExternalFunction
casADi__ExternalFunction__ExternalFunction' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__ExternalFunction__ExternalFunction_TIC x0' >>= (newForeignPtr c_delete_CasADi__ExternalFunction) >>= wrapReturn

-- classy wrapper
externalFunction' :: String -> IO ExternalFunction
externalFunction' = casADi__ExternalFunction__ExternalFunction'

